<?php


require_once("modules/Invoices/Invoice.php");

class InvoiceItems {
	var $id;
	var $bean = null;
	var $ss = null;
	var $items;
	var $tax_rate = 0;

	
	
	function InvoiceItems($id){
		$this->id = $id;
		$this->bean = new Invoice();
		$this->bean->retrieve($id);
	}
	

	function saveItems($arr){
		$this->bean->items = base64_encode(serialize($arr));		
		$this->bean->save();
	}
	

	function loadItems(){
		global $db;
		$re = $db->query("SELECT * FROM invoices WHERE id = '".$this->id."'");
		$ro = $db->fetchByAssoc($re);
		$arr = unserialize(base64_decode($ro['items']));
		if(is_array($arr))			
			$this->items = $arr;
		else
			$this->items = array();
		$this->calculateAmounts();
		$this->tax_rate = 0;
		if(!empty($this->bean->tax_rate))	
			$this->tax_rate = $this->bean->tax_rate;	
	}
	

	function calculateAmounts(){
		foreach($this->items as $k => $v){
			$amount = $this->items[$k]["quantity"] * $this->items[$k]["price"];
			$this->items[$k]["amount"] = number_format($amount, 2, '.', '');
		}
	}

	function prepareFormats(){
		foreach($this->items as $k => $v){
			$this->items[$k]["amount"] = $this->toNum($this->items[$k]["amount"]);
			$this->items[$k]["price"] = $this->toNum($this->items[$k]["price"]);
		}
	}

	
	function getSubTotalAmount(){
		$subtotal = 0;
		foreach($this->items as $k => $v)
			$subtotal += $v["amount"];
		return number_format($subtotal, 2, '.', '');
	}
	
	function getDiscountAmount(){
		$discount = 0;
		foreach($this->items as $k => $v)
			$discount += $v["amount"] * $v["discount_rate"] / 100.0;
		return number_format($discount, 2, '.', '');
	}
	
	function getTaxAmount($total = null){
		if(is_null($total))
			$total = $this->getSubTotalAmount() - $this->getDiscountAmount();		
		$tax = $total * ($this->tax_rate / 100.0);		
		return number_format($tax, 2, '.', '');
	}	

	function getTotalAmount(){
		$total = 0;
		$total = $this->getSubTotalAmount() - $this->getDiscountAmount();
		$total = $total + $this->getTaxAmount($total);		
		return number_format($total, 2, '.', '');
	}


	function parseFields($text){
		require_once("modules/Accounts/Account.php");
		$a = new Account;
		$a->retrieve($this->bean->account_id);


		foreach($a->field_defs as $f){
			if($f['type'] == 'date')
				$a->$f['name'] = $GLOBALS['timedate']->swap_formats($a->$f['name'],$GLOBALS['timedate']->get_date_format(),$GLOBALS['sugar_config']['default_date_format']);
			if($f['type'] == 'link')
				continue;
			
			$text = str_replace("\$account_".$f['name']."",$a->$f['name'],$text);						
		}


		foreach($this->bean->field_defs as $f){
			if($f['type'] == 'date')
				$this->bean->$f['name'] = $GLOBALS['timedate']->swap_formats($this->bean->$f['name'],$GLOBALS['timedate']->get_date_format(),$GLOBALS['sugar_config']['default_date_format']);
			if($f['type'] == 'link')
				continue;
			$text = str_replace("\$".$f['name']."",$this->bean->$f['name'],$text);						
		}	

		return $text;			
	}
	

	function getPDF(){		
		require_once("modules/Invoices/tcpdf/tcpdf.php");		
		$this->loadItems();

		$pdf = new TCPDF();		

		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetTitle($GLOBALS['mod_strings']['LBL_INVOICE']." ".$this->bean->invoice_number);
		$pdf->setPrintHeader(false);
				
		$pdf->AddPage();	
				
		$file = "custom/modules/Invoices/invoice_template.html";
		$html = "";
		if(file_exists($file))
			$html = file_get_contents($file)."<br>";
			

		$cursymb = $this->getCurrencySymbol();
		
		$html = $this->parseFields($html);
		

		$pdf->writeHTML($html, false, false, true, false, '');

		$pdf->SetLineWidth(0.2);
		

		$subtotal = $this->toNum($this->getSubTotalAmount());
		$discount = $this->toNum($this->getDiscountAmount());
		$tax = $this->toNum($this->getTaxAmount());
		$total = $this->toNum($this->getTotalAmount());
		
		

		if($this->bean->invoice_type == 'Hourly Rate'){
			$LBL_QUANTITY = $GLOBALS['mod_strings']['LBL_ITEM_HOURS'];
			$LBL_PRICE = $GLOBALS['mod_strings']['LBL_ITEM_RATE'];
		}else{
			$LBL_QUANTITY = $GLOBALS['mod_strings']['LBL_ITEM_QUANTITY'];
			$LBL_PRICE = $GLOBALS['mod_strings']['LBL_ITEM_PRICE'];
		}
		
		$w = array(110, 20, 30, 30);
		
		if($discount != 0)
			$w = array(90, 20, 30, 30,20);
		
		$pdf->SetFont('', 'B',10);
		$pdf->Cell($w[0], 6, $GLOBALS['mod_strings']['LBL_ITEM_NAME'], 'TLRB', 0, 'L', 0);
		$pdf->Cell($w[1], 6,$LBL_QUANTITY, 'RTB', 0, 'R', 0);
		$pdf->Cell($w[2], 6,$LBL_PRICE, 'RTB', 0, 'R', 0);
		$pdf->Cell($w[3], 6, $GLOBALS['mod_strings']['LBL_ITEM_AMOUNT'], 'RBT', 0, 'R', 0);	
		if($discount != 0)
			$pdf->Cell($w[4], 6, $GLOBALS['mod_strings']['LBL_ITEM_DISCOUNT'], 'RBT', 0, 'R', 0);	
			
		$pdf->Ln();
		$pdf->SetFont('', '');		
		foreach($this->items as $k => $item){
			$pdf->Cell($w[0], 6, $item["name"], 'LRB', 0, 'L', 0);
			$pdf->Cell($w[1], 6, $item["quantity"], 'RB', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $cursymb.$this->toNum($item["price"]), 'RB', 0, 'R', 0);
			$pdf->Cell($w[3], 6, $cursymb.$this->toNum($item["amount"]), 'RB', 0, 'R', 0);
			if($discount != 0){
				if($item["discount_rate"] != 0 && !empty($item["discount_rate"]))
					$pdf->Cell($w[4], 6, $item["discount_rate"]."%", 'RB', 0, 'R', 0);
				else
					$pdf->Cell($w[4], 6, "", 'RB', 0, 'R', 0);
			}		
			$pdf->Ln();
		}
		

		if($subtotal != $total){
			$pdf->Cell($w[0], 6, '', '', 0, 'R', 0);
			$pdf->Cell($w[1], 6, '', 'R', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $GLOBALS['mod_strings']['LBL_ITEM_SUBTOTAL'], 'RB', 0, 'R', 0);
			$pdf->Cell($w[3], 6, $cursymb.$subtotal, 'RTB', 0, 'R', 0);
			$pdf->Ln();
		}
		if($discount != 0){
			$pdf->Cell($w[0], 6, '', '', 0, 'R', 0);
			$pdf->Cell($w[1], 6, '', 'R', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $GLOBALS['mod_strings']['LBL_ITEM_DISCOUNT'], 'RB', 0, 'R', 0);
			$pdf->Cell($w[3], 6, $cursymb.$discount, 'BR', 0, 'R', 0);
			$pdf->Ln();
		}
		if($this->tax_rate != 0){
			$pdf->Cell($w[0], 6, '', '', 0, 'R', 0);
			$pdf->Cell($w[1], 6, '', 'R', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $GLOBALS['mod_strings']['LBL_ITEM_TAX_RATE'], 'RB', 0, 'R', 0);
			$pdf->Cell($w[3], 6, $this->tax_rate."%", 'RB', 0, 'R', 0);
			$pdf->Ln();		
			
			$pdf->Cell($w[0], 6, '', '', 0, 'R', 0);
			$pdf->Cell($w[1], 6, '', 'R', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $GLOBALS['mod_strings']['LBL_ITEM_TAX'], 'RB', 0, 'R', 0);
			$pdf->Cell($w[3], 6, $cursymb.$tax, 'RB', 0, 'R', 0);
			$pdf->Ln();
		}			
			$pdf->Cell($w[0], 6, '', '', 0, 'R', 0);
			$pdf->Cell($w[1], 6, '', 'R', 0, 'R', 0);
			$pdf->Cell($w[2], 6, $GLOBALS['mod_strings']['LBL_ITEM_TOTAL'], 'RB', 0, 'R', 0);
			$pdf->SetFont('', 'B');
			$pdf->Cell($w[3], 6, $cursymb.$total, 'RB', 0, 'R', 0);
			$pdf->SetFont('', '');
			$pdf->Ln();	
		
		return $pdf->Output(null,'S');			
	}

	
	
	function displayEdit(){
		$this->loadItems();	

		$this->ss = new Sugar_Smarty();
		
		$this->ss->assign('CURRENCY', $this->getCurrencySymbol());
			
		$this->ss->assign("MOD", $GLOBALS['mod_strings']);
		$this->ss->assign("APP", $GLOBALS['app_strings']);
		$this->ss->assign("ID", $this->id);		
		if($this->bean->invoice_type == 'Hourly Rate'){
			$this->ss->assign("LBL_QUANTITY",$GLOBALS['mod_strings']['LBL_ITEM_HOURS']);
			$this->ss->assign("LBL_PRICE",$GLOBALS['mod_strings']['LBL_ITEM_RATE']);
		}else{
			$this->ss->assign("LBL_QUANTITY",$GLOBALS['mod_strings']['LBL_ITEM_QUANTITY']);
			$this->ss->assign("LBL_PRICE",$GLOBALS['mod_strings']['LBL_ITEM_PRICE']);
		}
		
		if(!isset($select))
			$select = "";

		$this->ss->assign("SELECT_PROJECT_TASKS",$select);
		
		$this->ss->assign("ITEM_COUNT", count($this->items));
		$tax_rate = 0;
		if(!empty($this->bean->tax_rate))
			$tax_rate = $this->bean->tax_rate;
		$this->ss->assign("TAX_RATE", $tax_rate);
		$this->ss->assign("items", $this->items);

		return $this->ss->fetch('modules/Invoices/tpls/ItemsEdit.html');
		
	}

	

	function displayDetail(){
		$this->loadItems();		

		$this->ss = new Sugar_Smarty();

		$this->ss->assign('CURRENCY', $this->getCurrencySymbol());
		
		$this->ss->assign("MOD", $GLOBALS['mod_strings']);
		$this->ss->assign("APP", $GLOBALS['app_strings']);
		$this->ss->assign("ID", $this->id);		
		if($this->bean->invoice_type == 'Hourly Rate'){
			$this->ss->assign("LBL_QUANTITY",$GLOBALS['mod_strings']['LBL_ITEM_HOURS']);
			$this->ss->assign("LBL_PRICE",$GLOBALS['mod_strings']['LBL_ITEM_RATE']);
		}else{
			$this->ss->assign("LBL_QUANTITY",$GLOBALS['mod_strings']['LBL_ITEM_QUANTITY']);
			$this->ss->assign("LBL_PRICE",$GLOBALS['mod_strings']['LBL_ITEM_PRICE']);
		}
		
		$this->ss->assign("ITEM_COUNT", count($this->items));

		$this->ss->assign("SUBTOTAL_AMOUNT", $this->toNum($this->getSubTotalAmount()));
		$this->ss->assign("DISCOUNT_AMOUNT", $this->toNum($this->getDiscountAmount()));
		$this->ss->assign("TOTAL_AMOUNT", $this->toNum($this->getTotalAmount()));
		$this->ss->assign("TAX_RATE", $this->tax_rate);
		$this->ss->assign("TAX", $this->toNum($this->getTaxAmount()));

		$this->prepareFormats();
		$this->ss->assign("items", $this->items);
		
		if($this->bean->ACLAccess('Save'))
			$this->ss->assign("EDITABLE", 1);
		else
			$this->ss->assign("EDITABLE", 0);
		
		return $this->ss->fetch('modules/Invoices/tpls/ItemsDetail.html');
	}
	


	function getCurrencySymbol(){
		$currency = new Currency();
		if(isset($this->bean->currency_id) && !empty($this->bean->currency_id)){
			$currency->retrieve($this->bean->currency_id);
			if( $currency->deleted != 1)
				return $currency->symbol;
			else 
				return $currency->getDefaultCurrencySymbol();	
		}else
			return $currency->getDefaultCurrencySymbol();
	}

	static function toNum($n){
		return number_format($n,$GLOBALS['sugar_config']['default_currency_significant_digits'],$GLOBALS['sugar_config']['default_decimal_seperator'],$GLOBALS['sugar_config']['default_number_grouping_seperator']);
	}
		
}
?>
